/********************************************************************
 FileName:      HardwareProfile.h
 Dependencies:  See INCLUDES section
 Processor:     PIC18, PIC24, or PIC32 USB Microcontrollers
 Hardware:      The code is natively intended to be used on the 
                  following hardware platforms: 
                    PICDEM FS USB Demo Board
                    PIC18F46J50 FS USB Plug-In Module
                    PIC18F87J50 FS USB Plug-In Module
                    Explorer 16 + PIC24 or PIC32 USB PIMs
                    PIC24F Starter Kit
                    Low Pin Count USB Development Kit
                  The firmware may be modified for use on other USB 
                    platforms by editing this file (HardwareProfile.h)
 Compiler:  	Microchip C18 (for PIC18), C30 (for PIC24), 
                  or C32 (for PIC32)
 Company:       Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
  2.3   09/15/2008   Broke out each hardware platform into its own
                     "HardwareProfile - xxx.h" file
********************************************************************/

#ifndef HARDWARE_PROFILE_H
#define HARDWARE_PROFILE_H

//#define QUARZ_6MHZ
#define QUARZ_12MHZ
    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //#define USE_SELF_POWER_SENSE_IO	
    #define tris_self_power     TRISAbits.TRISA2    // Input
    #if defined(USE_SELF_POWER_SENSE_IO)
    #define self_power          PORTAbits.RA2
    #else
    #define self_power          1
    #endif

    //#define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISAbits.TRISA1    // Input
    #if defined(USE_USB_BUS_SENSE_IO)
    #define USB_BUS_SENSE       PORTAbits.RA1
    #else
    #define USB_BUS_SENSE       1
    #endif

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    //Uncomment the following line to make the output HEX of this 
    //  project work with the HID Bootloader
    #define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER	

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
    
    #define DEMO_BOARD LOW_PIN_COUNT_USB_DEVELOPMENT_KIT
    #define LOW_PIN_COUNT_USB_DEVELOPMENT_KIT
    //#define CLOCK_FREQ	48000000
	#define CLOCK_FREQ	24000000	// Emiliano
    
    /** LED ************************************************************/
	#define mInitAllLEDs()      {TRISCbits.TRISC6 = 0; TRISCbits.TRISC7 = 0;}

	#define mLED_Green			LATCbits.LATC6
	#define mLED_Red			LATCbits.LATC7

    #define mLED_Green_On()     mLED_Green = 1;
    #define mLED_Red_On()       mLED_Red = 1;
    
    #define mLED_Green_Off()    mLED_Green = 0;
    #define mLED_Red_Off()      mLED_Red = 0;
    
    #define mLED_Green_Toggle() mLED_Green = !mLED_Green;
    #define mLED_Red_Toggle()   mLED_Red = !mLED_Red;
  
    /** SWITCH *********************************************************/
    #define mSwitchUp			PORTCbits.RC5
    #define mSwitchDown			PORTCbits.RC4

	#define mInitAllSwitches() 	{TRISCbits.TRISC5 = 1; TRISCbits.TRISC4 = 1;}


    #define mEncA				PORTCbits.RC1
    #define mEncB				PORTCbits.RC2


    /** BUZZER *********************************************************/

#ifdef USE_SDA_BUZZER

	#define mBuzzer				LATBbits.LATB4
	#define mInitBuzzer()		{TRISBbits.TRISB4 = 0;}
	#define mReleaseBuzzer()	{TRISBbits.TRISB4 = 1; ANSELHbits.AN10 =0;}

#else

	#define mBuzzer				LATCbits.LATC5
	#define mInitBuzzer()		{TRISCbits.TRISC5 = 0;}
	#define mReleaseBuzzer()	{TRISCbits.TRISC5 = 1;}

#endif	// USE_SDA_BUZZER

    #define mBuzzer_On()       	mBuzzer = 1
    #define mBuzzer_Off()      	mBuzzer = 0

#ifdef USE_DIGITAL_ENC
	#define mInitENC()          {TRISCbits.TRISC2=1; TRISCbits.TRISC1=1;} // ANSELbits.AN5 =0; ANSELbits.AN6 =0;}

#else
    /** POT ************************************************************/
    //#define mInitPOT()          {TRISBbits.TRISB4=1;ADCON0=0x29;ADCON1=0;ADCON2=0x3E;ADCON2bits.ADFM = 1;}
	//#define mInitPOT()          {TRISCbits.TRISC2=1; ADCON0=0x19; ADCON1=0x05; ADCON2=0x3E; ADCON2bits.ADFM = 1;}
	//#define mInitPOT()          {TRISCbits.TRISC2=1; ADCON0=0x19; ADCON1=0x00; ADCON2=0x3E; ADCON2bits.ADFM = 1;}
	#define mInitPOT()            {TRISCbits.TRISC2=1; ADCON0=0x19; ADCON1=0x08; ADCON2=0x3E; ADCON2bits.ADFM=1; REFCON0=0xB0;}	//FVR voltage reference
#endif	
	#define mInitPOT_5V()         {TRISCbits.TRISC2=1; ANSEL |= 0x40; ADCON0=0x19; ADCON1=0x00; ADCON2=0x3E; ADCON2bits.ADFM=1; }	//VCC voltage reference

    /** TEMPERATURE (TC1047A)*******************************************/
	//#define mInitTemp()			{TRISCbits.TRISC3=1; ADCON0=0x1C; ADCON1=0x05; ADCON2=0x3E; ADCON2bits.ADFM = 1;}
	#define mInitTemp()			{TRISCbits.TRISC3=1; ADCON0=0x1C; ADCON1=0x08; ADCON2=0x3E; ADCON2bits.ADFM=1; REFCON0 =0xB0;} 	//FVR voltage reference

	/** LIGHT Sensor (TEMT6000)*******************************************/
	#define mInitLightSensor()	{TRISCbits.TRISC0=1; ADCON0=0x10; ADCON1=0x08; ADCON2=0x3E; ADCON2bits.ADFM=1; REFCON0 =0xB0;} 	//FVR voltage reference

#endif  //HARDWARE_PROFILE_H
