/********************************************************************
 FileName:     main.c

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
********************************************************************/

#ifndef USBMOUSE_C
#define USBMOUSE_C

/** INCLUDES *******************************************************/

#include "./USB/usb.h"
#include <delays.h>

#include "HardwareProfile.h"

#include "./USB/usb_function_hid.h"



/** VARIABLES ******************************************************/
//#pragma udata

/*
#if defined(__18F14K50) || defined(__18F13K50) || defined(__18LF14K50) || defined(__18LF13K50) 
    #pragma udata usbram2
#elif defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F4450) || defined(__18F2450)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#else
    #pragma udata
#endif
*/
extern unsigned char ReceivedDataBuffer[64];
extern unsigned char ToSendDataBuffer[64];

#pragma udata
extern USB_HANDLE USBOutHandle;
extern USB_HANDLE USBInHandle;



/** PRIVATE PROTOTYPES *********************************************/


#ifdef RINGTONES
#define MAX_NOTES 40

int default_divider;
int default_octave;
int beat_speed;
int melody_index, melody_count;
//unsigned char ROM Melody[] = {"16d5,16d#5,16d5,16d#5,16d5,16d#5,16d5,16d5,16d#5,16e5,16f5,16f#5,16g5,8g5,4p,8g5,4p,8a#5,8p,8c6,8p,8g5,4p,8g5,4p,8f5,8p,8p,8g5,4p,4p,8a#5,8p,8c6,8p,8g5,4p,4p,8f5,8p,8f#5,8p,8a#5,8g5,1d5"};
//#define DEFAULT_OCTAVE		6
//#define DEFAULT_DURATION	4
//#define BEAT_SPEED			150

typedef struct _NOTE
{
	UINT16  duration;
  	UINT16  frequency;
} NOTE;

BYTE play = 0;

NOTE melody[MAX_NOTES];

void PlayNote(UINT16 frequency, UINT16 duration);


void DoDelay(UINT32 ms_delay, UINT32 us_delay)
{
	while(ms_delay)
	{
		Delay1KTCYx(6);
		ms_delay--;
	}

	while(us_delay)
	{
		Nop();
		Nop();
		Nop();
//		Nop();
//		Nop();
		us_delay--;
	}
}

 void PlayNote(UINT16 frequency, UINT16 duration)
{
	UINT32 total_delay_time;                      // in microseconds
	UINT32 total_ms_delay_time, total_us_delay_time;
	UINT32 us_period, num_periods;

	if (0 == frequency)	// PAUSE
	{
		mBuzzer_Off();

		total_delay_time = (UINT32)duration;

		total_ms_delay_time = total_delay_time; // total delay time of ms
		total_us_delay_time = total_delay_time % 1000; // total delay time of us (ms already acounted for)

		DoDelay(total_ms_delay_time, total_us_delay_time);

		return;
	}

	us_period = 1000000ul / (UINT32)frequency;
	total_delay_time = us_period / 2; // calculate total delay time (10 for error)

	total_ms_delay_time = total_delay_time / 1000; // total delay time of ms
	total_us_delay_time = total_delay_time % 1000; // total delay time of us (ms already acounted for)
	
	num_periods = ((UINT32)duration * 1000) / us_period;

	num_periods >>= 2;

	while(num_periods)
	{
		DoDelay(total_ms_delay_time, total_us_delay_time);
		mBuzzer_On();

		DoDelay(total_ms_delay_time, total_us_delay_time);
		mBuzzer_Off();

		num_periods--;
	}
}

enum
{
	CMD_SET_OCTAVE      = 0x01,
	CMD_SET_DURATION,
	CMD_SET_BEAT,
	CMD_NOTE,
	CMD_PLAY,
	CMD_RESET,
	CMD_ERROR           = 0xFF
};


void SetNote(BYTE *pBuffer)
{
	BYTE octave;
	BYTE divider;
	BYTE i = 1; // pBuffer[0] --> command
	UINT32 tempo;

	if (melody_count >= (MAX_NOTES - 1))
	{
		return;
	}

	octave = default_octave;
	melody[melody_count].frequency = 0;

	switch (pBuffer[i])
	{
		case '6':
			if (pBuffer[i + 1] == '4')
			{
				divider = 64;
				i += 2;
			}
			break;
		case '3':
			if (pBuffer[i + 1] == '2')
			{
				divider = 32;
				i += 2;
			}
			break;
		case '1':
			if (pBuffer[i + 1] == '6')
			{
				divider = 16;
				i += 2;
			}
			else
			{
				divider = 1;
				i++;
			}
			break;
		case '8':
			divider = 8;
			i++;
			break;
		case '4':
			divider = 4;
			i++;
			break;
		case '2':
			divider = 2;
			i++;
			break;
		default:
			divider = default_divider;
	} 

	
	tempo = 60000 / beat_speed;   // milliseconds
	melody[melody_count].duration = (tempo / divider) * 4;
	
	if (pBuffer[i + 1] == '#')
	{
		switch (pBuffer[i])
		{
			case 'a':
				melody[melody_count].frequency = 466;
				break;
			case 'c':
				melody[melody_count].frequency = 277;
				break;
			case 'd':
				melody[melody_count].frequency = 311;
				break;
			case 'f':
				melody[melody_count].frequency = 370;
				break;
			case 'g':
				melody[melody_count].frequency = 415;
				break;
			default:
				melody[melody_count].frequency = 0;
				break;
		}
	
	i +=2;
	}
	else
	{
		switch (pBuffer[i])
		{
			case 'a':
				melody[melody_count].frequency = 440;
				break;
			case 'b':
				melody[melody_count].frequency = 494;
				break;
			case 'c':
				melody[melody_count].frequency = 262;
				break;
			case 'd':
				melody[melody_count].frequency = 294;
				break;
			case 'e':
				melody[melody_count].frequency = 330;
				break;
			case 'f':
				melody[melody_count].frequency = 349;
				break;
			case 'g':
				melody[melody_count].frequency = 392;
				break;
			case 'p':
				melody[melody_count].frequency = 0;
				break;
			default:
				melody[melody_count].frequency = 0;
				break;
		}
	
		i++;
	}
	
	if (pBuffer[i] == '.')
	{
		// Duration 1.5x
		melody[melody_count].duration += (melody[melody_count].duration >> 1);
		i++;
	}
	
	switch (pBuffer[i])
	{
		case '4': 
		case '5': 
		case '6': 
		case '7':
				octave = pBuffer[i] -'0';
				i++;
				break;
//		default:
	}
	
	if (octave >= 3)
		octave -= 3;
	else
		octave = 0;
	
	if (pBuffer[i] == '.')
	{
		// Duration 1.5x
		melody[melody_count].duration += (melody[melody_count].duration >> 1);
		i++;
	}
	
	melody[melody_count].frequency <<= octave;
	
	melody_count++;
}

void RingtonesPlayerTask(void)
{
	BOOL send_data = FALSE;

    if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
    {   
		memset(ToSendDataBuffer, 0, 8);

		switch(ReceivedDataBuffer[0])
		{
			case CMD_SET_OCTAVE:
				default_octave = (int)ReceivedDataBuffer[1];
				ToSendDataBuffer[0] = CMD_SET_OCTAVE;
				ToSendDataBuffer[1] = (BYTE)(default_octave >> 8);
				ToSendDataBuffer[2] = (BYTE)(default_octave & 0x00FF);
				send_data = TRUE;
				break;
		
			case CMD_SET_DURATION:
				default_divider = (int)ReceivedDataBuffer[1];
				ToSendDataBuffer[0] = CMD_SET_DURATION;
				ToSendDataBuffer[1] = (BYTE)(default_divider >> 8);
				ToSendDataBuffer[2] = (BYTE)(default_divider & 0x00FF);
				send_data = TRUE;
				break;
		
			case CMD_SET_BEAT:
				beat_speed = ((int)(ReceivedDataBuffer[1]) << 8) + (int)ReceivedDataBuffer[2];
				ToSendDataBuffer[0] = CMD_SET_BEAT;
				ToSendDataBuffer[1] = (BYTE)(beat_speed >> 8);
				ToSendDataBuffer[2] = (BYTE)(beat_speed & 0x00FF);
				send_data = TRUE;
				break;
		
			case CMD_NOTE:
				ToSendDataBuffer[0] = CMD_NOTE;
				SetNote(ReceivedDataBuffer);
				ToSendDataBuffer[1] = (BYTE)(melody[melody_count - 1].frequency >> 8);
				ToSendDataBuffer[2] = (BYTE)(melody[melody_count - 1].frequency & 0x00FF);
				ToSendDataBuffer[3] = (BYTE)(melody[melody_count - 1].duration >> 8);
				ToSendDataBuffer[4] = (BYTE)(melody[melody_count - 1].duration & 0x00FF);
				send_data = TRUE;
				break;
		
			case CMD_PLAY:
				ToSendDataBuffer[0] = CMD_PLAY;
				melody_index = 0;
				play = 1;
				send_data = TRUE;
				break;
			
			case CMD_RESET:
				ToSendDataBuffer[0] = CMD_RESET;
				melody_count = melody_index = 0;
				send_data = TRUE;
				break;
		
			default:
				ToSendDataBuffer[0] = CMD_ERROR;
				send_data = TRUE;
				break;
		}
	
		if (send_data)
		{
			if(!HIDTxHandleBusy(USBInHandle))
			{
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
			}
		}

        //Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,8);
	}
	else if (play)
	{
		for (melody_index = 0; melody_index < melody_count; melody_index++)
		{
			//if (melody[melody_index].frequency)
			{
				PlayNote(melody[melody_index].frequency, melody[melody_index].duration);
			}
		}
	
		mBuzzer_Off();
		melody_count = 0;
		play = 0;
	}
}
