/********************************************************************
 FileName:     main.c


********************************************************************/

/** INCLUDES *******************************************************/
#include "./USB/usb.h"
#include <delays.h>
#include "HardwareProfile.h"
#include "./USB/usb_function_hid.h"

/** CONFIGURATION **************************************************/

        #if defined(QUARZ_6MHZ)
        	#pragma config CPUDIV = NOCLKDIV
        #elif defined(QUARZ_12MHZ)
        	#pragma config CPUDIV = CLKDIV2
        #else 
            #error NO quarz selected for the selected demo board.
        #endif
        #pragma config USBDIV = OFF
        #pragma config FOSC   = HS
        #pragma config PLLEN  = ON
        #pragma config FCMEN  = OFF
        #pragma config IESO   = OFF
        #pragma config PWRTEN = OFF
        #pragma config BOREN  = OFF
        #pragma config BORV   = 30
        #pragma config WDTEN  = OFF
        #pragma config WDTPS  = 32768
        #pragma config MCLRE  = OFF
        #pragma config HFOFST = OFF
        #pragma config STVREN = ON
        #pragma config LVP    = OFF
        #pragma config XINST  = OFF
        #pragma config BBSIZ  = OFF
        #pragma config CP0    = OFF
        #pragma config CP1    = OFF
        #pragma config CPB    = OFF
        #pragma config WRT0   = OFF
        #pragma config WRT1   = OFF
        #pragma config WRTB   = OFF
        #pragma config WRTC   = OFF
        #pragma config EBTR0  = OFF
        #pragma config EBTR1  = OFF
        #pragma config EBTRB  = OFF                                                  // CONFIG7H


/** VARIABLES ******************************************************/


#if defined(__18F14K50) || defined(__18F13K50) || defined(__18LF14K50) || defined(__18LF13K50) 
    #pragma udata usbram2
#elif defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F4450) || defined(__18F2450)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#else
    #pragma udata
#endif


unsigned char ReceivedDataBuffer[64];
unsigned char ToSendDataBuffer[64];

#pragma udata
BYTE enc_up 	= 0;
BYTE enc_down	= 0;

USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = TRUE;

unsigned char digital_enc_cnt = 0;

/** PRIVATE PROTOTYPES *********************************************/
void BlinkUSBStatus(void);
BOOL SwitchUpIsPressed(void);
BOOL SwitchDownIsPressed(void);
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
long ReadPOT(void);
//long ReadADC(void);
int ReadTemp(void);
int ReadLightSensor(void);
void PlayBeep(void);



/** VECTOR REMAPPING ***********************************************/
#if defined(__18CXX)
	//On PIC18 devices, addresses 0x00, 0x08, and 0x18 are used for
	//the reset, high priority interrupt, and low priority interrupt
	//vectors.  However, the current Microchip USB bootloader 
	//examples are intended to occupy addresses 0x00-0x7FF or
	//0x00-0xFFF depending on which bootloader is used.  Therefore,
	//the bootloader code remaps these vectors to new locations
	//as indicated below.  This remapping is only necessary if you
	//wish to program the hex file generated from this project with
	//the USB bootloader.  If no bootloader is used, edit the
	//usb_config.h file and comment out the following defines:
	//#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER
	//#define PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
	#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
	#else	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
	#endif
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);        // See c018i.c in your C18 compiler dir
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
	#endif
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
	void Remapped_High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)
	{
	     _asm goto YourLowPriorityISRCode _endasm
	}
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	//Note: If this project is built while one of the bootloaders has
	//been defined, but then the output hex file is not programmed with
	//the bootloader, addresses 0x08 and 0x18 would end up programmed with 0xFFFF.
	//As a result, if an actual interrupt was enabled and occured, the PC would jump
	//to 0x08 (or 0x18) and would begin executing "0xFFFF" (unprogrammed space).  This
	//executes as nop instructions, but the PC would eventually reach the REMAPPED_RESET_VECTOR_ADDRESS
	//(0x1000 or 0x800, depending upon bootloader), and would execute the "goto _startup".  This
	//would effective reset the application.
	
	//To fix this situation, we should always deliberately place a 
	//"goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS" at address 0x08, and a
	//"goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS" at address 0x18.  When the output
	//hex file of this project is programmed with the bootloader, these sections do not
	//get bootloaded (as they overlap the bootloader space).  If the output hex file is not
	//programmed using the bootloader, then the below goto instructions do get programmed,
	//and the hex file still works like normal.  The below section is only required to fix this
	//scenario.
	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#endif	//end of "#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER)"

	#pragma code
	
	
	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
        #if defined(USB_INTERRUPT)
	        USBDeviceTasks();
        #endif
	     
		//Digital Encoder
		if (INTCON3bits.INT1IF)
		{
		    INTCON3bits.INT1IF = 0;        // Clear flag
		     if (!mEncB)
		     {
			  	enc_up = 1;   
			    if (digital_enc_cnt < 64u)
				     digital_enc_cnt ++;
		     	if(blinkStatusValid)
		     	  	mLED_Red_Toggle();
			  }
			  else
			  {
			  	enc_down = 1;   
			    if (digital_enc_cnt)
				     digital_enc_cnt --;
		    	if(blinkStatusValid)
		     	  mLED_Green_Toggle();
			  }   
		}
	
	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
	}	//This return will be a "retfie", since this is in a #pragma interruptlow section 
#endif




#pragma code


/********************************************************************
 * Function:        void PlayBeep(void)
 * Overview:        Plays a software beep
 *******************************************************************/
void PlayBeep(void)
{
	unsigned char i, j;

	//for (j = 0; j < 100; j++)
	{
	for (i = 0; i < 100u; i++)
	{
		mBuzzer_On();
		Delay10TCYx(75);

		mBuzzer_Off();
		Delay10TCYx(75);
	}
	}

	Delay1KTCYx(255);
}


/********************************************************************
 * Function:        void Check_Pot_Or_Enc(void)
 * Overview:        Checks for Potentiometer or Encoder.
 *******************************************************************/
BOOL Check_Pot_Or_Enc(void)	//1 if ENC
{
	WORD_VAL w;


	mInitPOT_5V();
	
	ADCON0bits.GO = 1;              // Start AD conversion
	while(ADCON0bits.NOT_DONE);     // Wait for conversion
	
	w.v[0] = ADRESL;
	w.v[1] = ADRESH;

	ANSEL &= ~0x40; //AN6 back to digital
	TRISCbits.TRISC2 = 1;  //back to input
	
	if (w.Val > 950u) //>4,6V!! with pot must be  0 < w.VAL < 4.1 
		return 1;
	
	//test for encoder to GND or 
	TRISCbits.TRISC2 = 0;
	LATCbits.LATC2 = 1;
	
	if (PORTCbits.RC2)	//r470 ohm present ->> POT!!
	{
		TRISCbits.TRISC2 = 1;  //back to input
		return 0;
	}
	else		//EncB was in gnd position!!
	{
		TRISCbits.TRISC2 = 1;  //back to input
		return 1;
	}
	
}

/********************************************************************
 * Function:        void main(void)
 * Overview:        Main program entry point.
 *******************************************************************/
void main(void)
{

	long num_periods;
	int i;

    InitializeSystem();
    
    
    Check_Pot_Or_Enc();

#ifdef USE_DIGITAL_ENC
	mInitENC();
    INTCON2bits.INTEDG1 = 0;        // Falling
    INTCON3bits.INT1IF = 0;        // Clear flag
    INTCON3bits.INT1IE = 1;        // Enable interrupt

	//INTCONbits.PEIE = 1;  		// enable global int
	INTCONbits.GIE = 1;  		// enable global int
#endif

	mInitBuzzer();

	PlayBeep();
 
    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
    {
        #if defined(USB_POLLING)
		// Check bus status and service USB interrupts.
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
        				  // this function periodically.  This function will take care
        				  // of processing and responding to SETUP transactions 
        				  // (such as during the enumeration process when you first
        				  // plug in).  USB hosts require that USB devices should accept
        				  // and process SETUP packets in a timely fashion.  Therefore,
        				  // when using polling, this function should be called 
        				  // regularly (such as once every 1.8ms or faster** [see 
        				  // inline code comments in usb_device.c for explanation when
        				  // "or faster" applies])  In most cases, the USBDeviceTasks() 
        				  // function does not take very long to execute (ex: <100 
        				  // instruction cycles) before it returns.
        #endif
    				  

		// Application-specific tasks.
		// Application related code may be added here, or in the ProcessIO() function.
        ProcessIO();        
    }//end while
}//end main

/******************************************************************************
 * Function:        void InitTimer(void)
 * Overview:        This routine perform Timers initialization
 *****************************************************************************/
void InitTimer(void)
{
/*
	T0CON = 0x08;				// Configure timer, but don't start it yet
	TMR0H = 0;
	TMR0L = BEAT_SPEED;			// Reset Timer0 to 0x0000
	INTCONbits.T0IF = 0;		// Clear Timer0 overflow flag
	INTCONbits.T0IE = 1;		// Enable Timer0 overflow interrupt
	T0CONbits.TMR0ON = 1;		// Start Timer0

	T1CON = 0b00000001;			// bit 7: 0 = 8 bit
								// bit 6: 0 = clock not derived from timer1 osc
								// bit 5-4: 00 = prescaler 1:1
								// bit 3: 0 = timer1 osc shut off
								// bit 2: 0 = not syncronize with ext clock
								// bit 1: 0 = internal osc
								// bit 0: 1 = timer1 start

    PIR1bits.TMR1IF = 0;        // Clear Timer1 overflow flag
    PIE1bits.TMR1IE = 1;        // Enable Timer1 overflow interrupt

	INTCONbits.PEIE = 1;  		// enable global int
	//INTCONbits.GIE = 1;  		// enable global int
*/
}

/********************************************************************
 * Function:        static void InitializeSystem(void)
  * Overview:       InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *******************************************************************/
static void InitializeSystem(void)
{
    #if (defined(__18CXX) & !defined(PIC18F87J50_PIM))
	ADCON1 |= 0x0F;                 // Default all pins to digital
    ANSEL &= ~0x60;                 // Default all pins to digital
    #elif defined(__C30__)
        AD1PCFGL = 0xFFFF;
    #elif defined(__C32__)
        AD1PCFG = 0xFFFF;
    #endif

    #if defined(PIC18F87J50_PIM) || defined(PIC18F46J50_PIM) || defined(PIC18F_STARTER_KIT_1) || defined(PIC18F47J53_PIM)
	//On the PIC18F87J50 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 48MHz
	//operation.  On these devices, user firmware needs to manually set the OSCTUNE<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        OSCTUNEbits.PLLEN = 1;  //Enable the PLL and wait 2+ms until the PLL locks before enabling USB module
        while(pll_startup_counter--);
    }
    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif

 
    
//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem



/******************************************************************************
 * Function:        void UserInit(void)
 * Overview:        This routine perform Unser initialization
 *****************************************************************************/
void UserInit(void)
{
    //Initialize all of the LED pins
    mInitAllLEDs();
    
    //Initialize all of the push buttons
    mInitAllSwitches();
    
    //initialize the variable holding the handle for the last
    // transmission
    USBOutHandle = 0;
    USBInHandle = 0;

    blinkStatusValid = TRUE;
}//end UserInit



#ifdef VOLUME_CONTROL
/******************************************************************************
 * Function:        void VolumeTask(void)
 * Overview:        This routine should take care of all of the Volume code
 *****************************************************************************/
void VolumeTask(void)
{
	static long old_volume;
	long volume;
	if (SwitchUpIsPressed() || enc_up)		// Volume Up
	{
        enc_up = 0;
        
		ToSendDataBuffer[0] = 0x01;
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
		Delay1KTCYx(120);	// delay 20 ms
		ToSendDataBuffer[0] = 0x00;
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
	}
	else if (SwitchDownIsPressed() || enc_down)	// Volume Down
	{
        enc_down = 0;

		ToSendDataBuffer[0] = 0x02;
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
		Delay1KTCYx(120);	// delay 20 ms
		ToSendDataBuffer[0] = 0x00;
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
	}

#ifdef USE_DIGITAL_ENC
	return;
#endif

	
	volume = ReadPOT();

	if (volume != old_volume)
	{
		if (volume > old_volume)
		{
			ToSendDataBuffer[0] = 0x01;
			USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
			Delay1KTCYx(120);	// delay 20 ms
			ToSendDataBuffer[0] = 0x00;
			USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
		}
		else 
		{
			ToSendDataBuffer[0] = 0x02;
			USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
			Delay1KTCYx(120);	// delay 20 ms
			ToSendDataBuffer[0] = 0x00;
			USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
		}

		old_volume = volume;
	}


}
#endif //VOLUME_CONTROL

#if defined JOYSTICK
/******************************************************************************
 * Function:        void JoystickTask(void)
 * Overview:        This routine should take care of all of the Joystick code
 *****************************************************************************/
void JoystickTask(void)
{
	enum
	{
		THROTTLE = 0,
		DIR_X,
		DIR_Y
	};
	static BYTE variable_id = THROTTLE;
	static BYTE variables[3]; // 0 = throttle, 1 = x, 2 = y, 3 = buttons
	static long old_pot;
	long pot;
	BOOL Datachanged = FALSE;

	pot = ReadPOT();
	if (old_pot != pot)
	{
		old_pot = pot;
		Datachanged = TRUE;
	}
	
	ToSendDataBuffer[3] = 0;
	
	if (!mSwitchUp)  		// Button 1 
	{
		ToSendDataBuffer[3] |= 0x01;
		Datachanged = TRUE;
	}
	if (!mSwitchDown) 		// Button 2
	{
		ToSendDataBuffer[3] |= 0x02;
		Datachanged = TRUE;
	}
	
	if	(Datachanged)
	{
		memcpy((void *)ToSendDataBuffer, (void *)variables, 3);
		variables[variable_id] = (int)((pot / 4) - 0x80);
		ToSendDataBuffer[variable_id] = variables[variable_id];

		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 4);
	}

	if (SwitchUpIsPressed())  // Button 1 (select variable)
	{
		variable_id++;
		if (variable_id > DIR_Y)
			variable_id = THROTTLE;
	}
	
//	if (send_reset)
	if (mSwitchDown && mSwitchUp)
	{
		Delay10KTCYx(120);	// delay 250 ms
		ToSendDataBuffer[3] = 0x00;
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 4);
	}
}
#endif //JOYSTICK

#if defined MEDIA_PLAYER
/******************************************************************************
 * Function:        void MediaPlayerTask(void)
 * Overview:        This routine should take care of all of the MediaPlayer code
 *****************************************************************************/
void MediaPlayerTask(void)
{
	enum
	{
		NEXT_TRACK = 0x01,
		PREV_TRACK = 0x02,
		STOP       = 0x04,
		PLAY_PAUSE = 0x08,
		MUTE       = 0x10,
		VOL_UP     = 0x20,
		VOL_DOWN   = 0x40
	};
	static int old_pot;
	int pot;
	BOOL send_data = FALSE;
	
	ToSendDataBuffer[0] = 0;

#ifndef USE_DIGITAL_ENC

	if (enc_up)      // Volume Up
	{
        enc_up = 0;
		ToSendDataBuffer[0] |= VOL_UP;
		send_data = TRUE;
	}

	if (enc_down)       // Volume Down
	{
        enc_down = 0;
		ToSendDataBuffer[0] |= VOL_DOWN;
		send_data = TRUE;
	}
#else
	pot = ReadPOT();
	
	if ((old_pot - pot) > 50)           // Volume Down
	{
		ToSendDataBuffer[0] |= VOL_DOWN;
		send_data = TRUE;
		old_pot = pot;
	}
	else if ((pot - old_pot) > 50)      // Volume Up
	{
		ToSendDataBuffer[0] |= VOL_UP;
		send_data = TRUE;
		old_pot = pot;
	}

#endif

	if (SwitchUpIsPressed())      // Play / Pause
	{
		if (!mSwitchDown) // Both Prev Track
			ToSendDataBuffer[0] |= PREV_TRACK;
		else
			ToSendDataBuffer[0] |= PLAY_PAUSE;
		send_data = TRUE;
	}
	else if (SwitchDownIsPressed()) // Next Track
	{
		ToSendDataBuffer[0] |= NEXT_TRACK;
		send_data = TRUE;
	}
	
	if (send_data)
	{	
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
		Delay1KTCYx(120);	// delay 20 ms
		ToSendDataBuffer[0] = 0x00;       // Reset
		USBInHandle = HIDTxPacket(HID_EP, (BYTE *)&ToSendDataBuffer[0], 1);
	}
}

#endif //MEDIA_PLAYER

#if defined HIDDEMO
/******************************************************************************
 * Function:        void HID_DEMO_Task(void)
 * Overview:        This routine should take care of all of the HID_DEMO_Task code
 *****************************************************************************/
void HID_DEMO_Task(void)
{
	enum
	{
	    CMD_GET_TEMPERATURE = 0x51,
	    CMD_GET_POT         = 0x52,
	    CMD_CONTROL_LED     = 0x53,
	    CMD_FREE_LED        = 0x54,
		CMD_GET_LIGHTSENSOR = 0x55,
		CMD_GET_BUTTONS		= 0x56,
	    GREEN_LED_OFF       = 0x60,
	    GREEN_LED_ON        = 0x61,
	    RED_LED_OFF         = 0x62,
	    RED_LED_ON          = 0x63,
	    CMD_ERROR           = 0xFF
	};
	int value;
	
    if(HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
			return;
			
	switch(ReceivedDataBuffer[0])				//Look at the data the host sent, to see what kind of application specific command it sent.
	{
		case CMD_GET_TEMPERATURE:
			value = ReadTemp();
			ToSendDataBuffer[0] = CMD_GET_TEMPERATURE;
			ToSendDataBuffer[1] = (int)((value & 0xFF00) >> 8);
			ToSendDataBuffer[2] = (int)((value & 0x00FF));
	
			if(!HIDTxHandleBusy(USBInHandle))
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
			break;
	
		case CMD_GET_LIGHTSENSOR:
			value = ReadLightSensor();
			ToSendDataBuffer[0] = CMD_GET_LIGHTSENSOR;
			ToSendDataBuffer[1] = (int)((value & 0xFF00) >> 8);
			ToSendDataBuffer[2] = (int)((value & 0x00FF));
	
			if(!HIDTxHandleBusy(USBInHandle))
					USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
			break;
	
		case CMD_GET_BUTTONS:
			ToSendDataBuffer[0] = CMD_GET_BUTTONS;
			ToSendDataBuffer[1] = !mSwitchUp;
			ToSendDataBuffer[2] = !mSwitchDown;
	
			if(!HIDTxHandleBusy(USBInHandle))
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
			break;
	
		case CMD_GET_POT:
			value = ReadPOT();
			ToSendDataBuffer[0] = CMD_GET_POT;
			ToSendDataBuffer[1] = (int)((value & 0xFF00) >> 8);
			ToSendDataBuffer[2] = (int)((value & 0x00FF));
	
			if(!HIDTxHandleBusy(USBInHandle))
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
			break;
	
		case CMD_CONTROL_LED:
			ToSendDataBuffer[0] = CMD_CONTROL_LED;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				blinkStatusValid = FALSE;			//Stop blinking the LEDs automatically, going to manually control them now.
			}
			break;
	
		case CMD_FREE_LED:
			ToSendDataBuffer[0] = CMD_FREE_LED;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				blinkStatusValid = TRUE;			//Start blinking the LEDs automatically, going to manually control them now.
			}
			break;
	
		case GREEN_LED_OFF:
			ToSendDataBuffer[0] = GREEN_LED_OFF;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				if (!blinkStatusValid)
					mLED_Green_Off();
			}
			break;
	
		case GREEN_LED_ON:
			ToSendDataBuffer[0] = GREEN_LED_ON;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				if (!blinkStatusValid)
					mLED_Green_On();
			}
			break;
	
		case RED_LED_OFF:
			ToSendDataBuffer[0] = RED_LED_OFF;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				if (!blinkStatusValid)
					mLED_Red_Off();
			}
			break;
	
		case RED_LED_ON:
			ToSendDataBuffer[0] = RED_LED_ON;
			if(!HIDTxHandleBusy(USBInHandle))
		             {
				USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0], 8);
				if (!blinkStatusValid)
					mLED_Red_On();
			}
			break;
		
		default:
			break;
/*
           case 0x80:  //Toggle LEDs command
	        blinkStatusValid = FALSE;			//Stop blinking the LEDs automatically, going to manually control them now.
               break;
           case 0x81:  //Get push button state
               ToSendDataBuffer[0] = 0x81;				//Echo back to the host PC the command we are fulfilling in the first byte.  In this case, the Get Pushbutton State command.

               if(!HIDTxHandleBusy(USBInHandle))
               {
                   USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
               }
               break;

           case 0x37:	//Read POT command.  Uses ADC to measure an analog voltage on one of the ANxx I/O pins, and returns the result to the host
               {
                   WORD_VAL w;

                if(!HIDTxHandleBusy(USBInHandle))
                {
                    mInitPOT();
                    w = ReadPOT();					//Use ADC to read the I/O pin voltage.  See the relevant HardwareProfile - xxxxx.h file for the I/O pin that it will measure.
													//Some demo boards, like the PIC18F87J50 FS USB Plug-In Module board, do not have a potentiometer (when used stand alone).
													//This function call will still measure the analog voltage on the I/O pin however.  To make the demo more interesting, it
													//is suggested that an external adjustable analog voltage should be applied to this pin.
					ToSendDataBuffer[0] = 0x37;  	//Echo back to the host the command we are fulfilling in the first byte.  In this case, the Read POT (analog voltage) command.
					ToSendDataBuffer[1] = w.v[0];  	//Measured analog voltage LSB
					ToSendDataBuffer[2] = w.v[1];  	//Measured analog voltage MSB

                    USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
                }					
               }
               break;
*/
    }
	//Re-arm the OUT endpoint for the next packet
	USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,8);
} 
#endif // HIDDEMO

/********************************************************************
 * Function:        void ProcessIO(void)
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *******************************************************************/
void ProcessIO(void)
{   
/*
	value = ReadPOT();
	noteFrequency = (int)(value >> 8);
	PlayBeep();
*/
    //Blink the LEDs according to the USB device status
    if(blinkStatusValid)
        BlinkUSBStatus();

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE) || USBSuspendControl ) 
    	return;

#ifdef VOLUME_CONTROL

	VolumeTask();

#elif defined JOYSTICK

	JoystickTask();

#elif defined MEDIA_PLAYER

	MediaPlayerTask();

#elif defined RINGTONES

	RingtonesPlayerTask();

#elif defined HIDDEMO

   HID_DEMO_Task();

#else
 	#error "NO PROFILE selected in USB_CONFIG.H"
#endif

}//end ProcessIO


/******************************************************************************
 * Function:        BOOL SwitchUpIsPressed(void)
 * Output:          TRUE - pressed, FALSE - not pressed
 * Overview:        Indicates if the switch is pressed.  
 *****************************************************************************/
BOOL SwitchUpIsPressed(void)
{
	static BYTE old_SwitchUp = 1;
    if(mSwitchUp != old_SwitchUp)
    {
        old_SwitchUp = mSwitchUp;		// Save new value
        if(!mSwitchUp)      	   		// If pressed
            return TRUE;                // Was pressed
    }
    return FALSE;                       // Was not pressed
}

/******************************************************************************
 * Function:        BOOL SwitchDownIsPressed(void)
 * Output:          TRUE - pressed, FALSE - not pressed
 * Overview:        Indicates if the switch is pressed.  
 *****************************************************************************/
BOOL SwitchDownIsPressed(void)
{
	static BYTE old_SwitchDown = 1;
   	if(mSwitchDown != old_SwitchDown)
    {
        old_SwitchDown = mSwitchDown;	// Save new value
        if(!mSwitchDown)            	// If pressed
            return TRUE;                // Was pressed
    }
    return FALSE;                       // Was not pressed
}

/******************************************************************************
 * Function:        long ReadADC(void)
 * Output:          long - the 10-bit right justified ADC value
 * Overview:        This function reads the ADC 
 *****************************************************************************/
long ReadADC(void)
{
  	WORD_VAL ADC_Value;
  	
  	ADCON0bits.ADON = 1;        	// Enable the A/D
    ADCON0bits.GO = 1;              // Start AD conversion
    while(ADCON0bits.NOT_DONE);     // Wait for conversion

  	ADC_Value.v[0] = ADRESL;  	// Read ADRESL into the lower byte
  	ADC_Value.v[1] = ADRESH;  	// Read ADRESH into the high byte

  	return (ADC_Value.Val);     	// Return the long variable
}


/******************************************************************************
 * Function:        long ReadPOT(void)
 * Output:          long - the 10-bit right justified POT value
 * Overview:        This function reads the POT 
 *****************************************************************************/
long ReadPOT(void)
{
    long POT_Value;

#ifdef USE_DIGITAL_ENC
	POT_Value = (long)digital_enc_cnt * 16;
	if (POT_Value > 1023)
		return 1023;
	else
		return (POT_Value);
#else
	mInitPOT();
	
	POT_Value = 1023 - ReadADC();
	
    return (POT_Value);
#endif
}

/******************************************************************************
 * Function:        int ReadTemp(void)
 * Output:          int temperature decimi di C
 * Overview:        This function reads the temperature 
 *****************************************************************************/
int ReadTemp(void)
{
	long temperature;

	mInitTemp();

	temperature = ReadADC();
	temperature += ReadADC();
	temperature += ReadADC();
	temperature += ReadADC();

	temperature *= 4096;		//FVR voltage [mV]
	temperature /= (4 * 1024);  //meand on 4read - 1024 = 10 bit
	temperature -= 500;			//sensor offset [mV]

	return ((int)temperature);	//in decimi di grado
}


/******************************************************************************
 * Function:        int ReadLightSensor(void)
 * Output:          int 
 * Overview:        This function reads the luminosity 
 *****************************************************************************/
int ReadLightSensor(void)
{
	long luminosity;
	int i;

	mInitLightSensor();

	luminosity = 0;
	for (i = 0; i < 4; i++) 
	{
		luminosity += ReadADC();
	}

	return ((int)(luminosity >> 2));	// luminosity / 4
}

/********************************************************************
 * Function:        void BlinkUSBStatus(void)
 * Overview:        BlinkUSBStatus turns on and off LEDs 
 *                  corresponding to the USB device state.
 *
 * Note:            mLED macros can be found in HardwareProfile.h
 *                  USBDeviceState is declared and updated in
 *                  usb_device.c.
 *******************************************************************/
void BlinkUSBStatus(void)
{
    static WORD led_count=0;
    
    if(--led_count == 0u)
    	led_count = 10000u;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On(); mLED_2_On(); }
    #define mLED_Only_1_On()        {mLED_1_On(); mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On(); }

    if(USBSuspendControl)
    {
        if(led_count==0)
            mLED_Green_Toggle();
    }
    else
    {
	    switch (USBDeviceState)
	    {
	    	case DETACHED_STATE:
	    			mLED_Green_Off();
            		mLED_Red_Off();
            		break;
 	    	case ATTACHED_STATE:
	    			mLED_Green_On();
            		mLED_Red_On();
            		break;
 	    	case POWERED_STATE:
	    			mLED_Green_On();
            		mLED_Red_Off();
            		break;
 	    	case DEFAULT_STATE:
	    			mLED_Green_Off();
            		mLED_Red_On();
            		break;
	    	case ADDRESS_STATE:
					if(led_count == 0u)
					{
					    mLED_Green_Toggle();
					    mLED_Red_Off();
					} 
			   		break;
	    	case CONFIGURED_STATE:
					if(led_count == 0u)
					{
					    mLED_Green_Toggle();
					} 
			   		break;
	    }

    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus




// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 * Overview:        Call back that is invoked when a USB suspend is detected
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

    #if defined(__C30__)
        _IPL = 7;
        #if defined(__dsPIC33EP512MU810__)||defined(__PIC24EP512GU810__)
        #else
        USBSleepOnSuspend();
        #endif
        _IPL = 0;
    #endif
}

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *******************************************************************/
void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function will only be able to wake up the host if
 *                  all of the below are true:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *                  If the host has not armed the device to perform remote wakeup,
 *                  then this function will return without actually performing a
 *                  remote wakeup sequence.  This is the required behavior, 
 *                  as a USB device that has not been armed to perform remote 
 *                  wakeup must not drive remote wakeup signalling onto the bus;
 *                  doing so will cause USB compliance testing failure.
 *                  
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            This function does nothing and returns quickly, if the USB
 *                  bus and host are not in a suspended condition, or are 
 *                  otherwise not in a remote wakeup ready state.  Therefore, it
 *                  is safe to optionally call this function regularly, ex: 
 *                  anytime application stimulus occurs, as the function will
 *                  have no effect, until the bus really is in a state ready
 *                  to accept remote wakeup. 
 *
 *                  When this function executes, it may perform clock switching,
 *                  depending upon the application specific code in 
 *                  USBCBWakeFromSuspend().  This is needed, since the USB
 *                  bus will no longer be suspended by the time this function
 *                  returns.  Therefore, the USB module will need to be ready
 *                  to receive traffic from the host.
 *
 *                  The modifiable section in this routine may be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of ~3-15 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at least 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    }while(delay_count);
    USBResumeControl = 0;
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch( (INT)event )
    {
        case EVENT_TRANSFER:
            //Add application specific callback task or callback function here if desired.
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER_TERMINATED:
            //Add application specific callback task or callback function here if desired.
            //The EVENT_TRANSFER_TERMINATED event occurs when the host performs a CLEAR
            //FEATURE (endpoint halt) request on an application endpoint which was 
            //previously armed (UOWN was = 1).  Here would be a good place to:
            //1.  Determine which endpoint the transaction that just got terminated was 
            //      on, by checking the handle value in the *pdata.
            //2.  Re-arm the endpoint if desired (typically would be the case for OUT 
            //      endpoints).
            break;
        default:
            break;
    }      
    return TRUE; 
}

